
//**************************************************************************************************/
#include <__EthEnc28j60.h>

/**************************************************************************************************
*
**************************************************************************************************/

// duplex config flags
#define Spi_Ethernet_HALFDUPLEX     0x00  // half duplex
#define Spi_Ethernet_FULLDUPLEX     0x01  // full duplex

// mE ehternet NIC pinout
sfr sbit SPI_Ethernet_Rst at LATF0_bit;  // for writing to output pin always use latch
sfr sbit SPI_Ethernet_CS  at LATF1_bit;  // for writing to output pin always use latch
sfr sbit SPI_Ethernet_Rst_Direction at TRISF0_bit;
sfr sbit SPI_Ethernet_CS_Direction  at TRISF1_bit;
// end ethernet NIC definitions


//ROM constant strings
const unsigned char httpHeader[] = "HTTP/1.1 200 OK\nContent-type: ";            // HTTP header
const unsigned char httpMimeTypeHTML[]= "text/html; charset=iso-8859-7\n\n";     // HTML MIME type
const unsigned char httpMimeTypeScript[] = "text/plain\n\n";                     // TEXT MIME type
unsigned char httpMethod[] = "GET /";



// RAM variables
unsigned char   myMacAddr[6] = {0x00, 0x14, 0xA5, 0x76, 0x19, 0x3f};   // my MAC address
unsigned char   myIpAddr[4]  = {192, 168,  0, 70 };                   // my IP address
unsigned char   gwIpAddr[4]  = {192, 168,  0,  1 };                   // gateway (router) IP address
unsigned char   ipMask[4]    = {255, 255, 255,  0 };                   // network mask (for example : 255.255.255.0)
unsigned char   dnsIpAddr[4] = {192, 168,  0,  1 };                   // DNS server IP address

unsigned char   getRequest[15];                                        // HTTP request buffer
unsigned char   dyna[30];                                              // buffer for dynamic response
unsigned long   httpCounter = 0;                                       // counter of HTTP requests

// my project specific variables
unsigned long  eeAddr = 0x7FFC30;                                     // Start address of EEPROM


int AirSpeedTick=0;
int AirSpeed=0;
int AirSpeedCoficient=9;
int PreviusAirSpeed=0;
int AverageAirSpeed=0;
int MaxAirSpeed=0;
int MinAirSpeed=0;
int ZeroSpeedCounter=0;

int TickCounter=0 ;  // counts 4 250ms for 1 second
int minutes=0;     // averges are calculated every 10 minutes
                   // Temperature
int Temp[4];         // general vars
int TempTick=0;      // counts 5 sec to get Temp and hum from DHT11
char integTemp=0;
char decimTemp=0;
                     //Humidity
char integHum=0;
char decimHum=0;
char checksum=0;

 char txt[4];

int LastQuarterSpeed[20];
 
// Air derection vars
int ALeds[4];
unsigned char Dlogo[]="-----";


#define putConstString  SPI_Ethernet_putConstString

#define putString  SPI_Ethernet_putString

/*
 * this function is called by the library
 * the user accesses to the HTTP request by successive calls to Spi_Ethernet_getByte()
 * the user puts data in the transmit buffer by successive calls to Spi_Ethernet_putByte()
 * the function must return the length in bytes of the HTTP reply, or 0 if nothing to transmit
 *
 * if you don't need to reply to HTTP requests,
 * just define this function with a return(0) as single statement
 *
 */
unsigned int  SPI_Ethernet_UserTCP(unsigned char *remoteHost, unsigned int remotePort,
                               unsigned int localPort, unsigned int reqLength, TEthPktFlags *flags) {
  unsigned int    len = 0 ;                   // my reply length
  unsigned int    i ;                         // general purpose integer

  // should we close tcp socket after response is sent?
  // library closes tcp socket by default if canClose flag is not reset here
  // flags->canClose = 0; // 0 - do not close socket
                          // otherwise - close socket

  if(localPort != 90) {                        // I listen only to web request on port 90
    return(0) ;
  }

  // get 10 first bytes only of the request, the rest does not matter here
  for(i = 0 ; i < 10 ; i++) {
    getRequest[i] = SPI_Ethernet_getByte() ;
  }
  getRequest[i] = 0 ;

  if(memcmp(getRequest, httpMethod, 5)) {      // only GET method is supported here
    return(0) ;
  }

  httpCounter++ ;                              // one more request done

  if(getRequest[5] == 's') {                   // if request path name starts with s, store dynamic data in transmit buffer
    // the text string replied by this request can be interpreted as javascript statements
    // by browsers

    len = putConstString(httpHeader) ;         // HTTP header
    len += putConstString(httpMimeTypeScript); // with text MIME type
    
    // add AN2 value to reply
    //ALeds[0]=ADC1_Read(0);
    IntToStr(ALeds[0], dyna) ;    len += putConstString("RB0") ;    len += putString(dyna) ;
    InttoStr(ADC1_Read(0),dyna); len += putString("  Real value = " ) ; len += putString(dyna) ;
    len += putConstString(";\n") ;

    //Temp[1]=ADC1_Read(1);
    IntToStr(ALeds[1], dyna) ;    len += putConstString("RB1") ;    len += putString(dyna) ;
    InttoStr(ADC1_Read(01),dyna); len += putString("  Real value = " ) ; len += putString(dyna) ;
    len += putConstString(";\n") ;

    //Temp[2]=ADC1_Read(2);
    IntToStr(ALeds[2], dyna) ;    len += putConstString("RB2") ;    len += putString(dyna) ;
    InttoStr(ADC1_Read(02),dyna); len += putString("  Real value = " ) ; len += putString(dyna) ;
    len += putConstString(";\n") ;
    
    //Temp[3]=ADC1_Read(3);
    IntToStr(ALeds[3], dyna) ;    len += putConstString("RB3") ;    len += putString(dyna) ;
    InttoStr(ADC1_Read(3),dyna); len += putString("  Real value = " ) ; len += putString(dyna) ;
    len += putConstString(";\n") ;
    
      //Air Speed tick
    IntToStr(AirSpeed, dyna) ;    len += putConstString("Air Speed Tick") ;    len += putString(dyna) ;
    InttoStr(AirSpeedTick,dyna); len += putString("value = " ) ; len += putString(dyna) ;
    len += putConstString(";\n") ;

      //AirSpeed coficient
      len += putConstString("Air Speed Cof value = ") ;  InttoStr(AirSpeedCoficient,dyna);  len += putString(dyna) ;
    len += putConstString(";\n") ;

      //AirSpeed
    len += putConstString("AirSpeed value = ") ; IntToStr(AirSpeed, dyna) ;   len += putString(dyna) ;
    len += putConstString(";\n") ;

       //Temperature
    ByteToStr(integTemp, dyna) ;    len += putConstString("Temp ") ;    len += putString(dyna) ;
         ByteToStr(decimTemp, dyna) ;    len += putConstString(",") ;    len += putString(dyna) ;
    len += putConstString(";\n") ;


       //Humidity
    ByteToStr(integHum, dyna) ;    len += putConstString("Humidity ") ;    len += putString(dyna) ;
         ByteToStr(decimHum, dyna) ;    len += putConstString(",") ;    len += putString(dyna) ;
    len += putConstString(";\n") ;
    
    
    // add HTTP requests counter to reply
    IntToStr(httpCounter, dyna) ;
    len += putConstString("  REQ=") ;
    len += putString(dyna) ;
    
    len += putConstString("\n to change Air Speed cof:  URL/t18\n") ;
    len += putConstString(";") ;
  }

  else if(getRequest[5] == 't') {             // if request path name starts with t, store value to AirSpeedCoficient
    unsigned char  AirCof[20];
    int NumAirCof=0;
    len = putConstString(httpHeader) ;         // HTTP header
    len += putConstString(httpMimeTypeScript); // with text MIME type

    if(isdigit(getRequest[6])) {              // if 0 <= bit number <= 9,
       Aircof[0]=getRequest[6] ;              // convert ASCII to integer
       AirCof[1]=getRequest[7];
       NumAirCof=atoi(AirCof);
       AirSpeedCoficient=NumAircof;           // uodate with new value
       EEPROM_Write(eeAddr,NumAirCof);
       len += putConstString("AirSpeedCoficient written in eeprom is = ") ;
       for (i=9;i<10;i++){
             intToStr(NumAirCof, dyna) ;  len += putString(dyna) ;

            }


 }else{
           len += putConstString("AirSpeedCoficient not set") ;
       }
    len += putConstString("----\n") ;
  }


  if(len == 0) {                              // what do to by default

    len=putConstString("HTTP/1.1 200 OK\n Content-type:text/html; charset=UTF-8 \n\n <html><body>");
  //  len += putConstString(httpMimeTypeScript); // with text MIME type

     len += putConstString("<p mode=\"wrap\" align=\"center\"> <a href=/>Reload</a></p>");
      len += putConstString("<table border=\"1\" bordercolor=\"black\" cellspacing=\"2\" cellpadding=\"2\" height=\"100\" align=\"center\">");
      len += putConstString("<tr bordercolor=\"#0000CC\" bgcolor=\"#948ACC\" style=\"font: normal 12pt Arial; text-align: center\">");
      len += putConstString("<td></td>");     //temp
      len += putConstString("<td></td>");          //wind
      len += putConstString("<td></td>");       //direction
      len += putConstString("<td></td>");         //humidity
      len += putConstString("</tr>");
      len += putConstString("<tr bordercolor=\"#0000CC\" style=\"font: normal  bold 14pt Arial; text-align: center\">");
      len += putConstString("<td>");ByteToStr(integTemp,txt);len += putString(txt);//len += putString(",");ByteToStr(decimTemp,txt);len += putString(txt);
                           len += putString(" C</td>");    // temperature
      len += putConstString("<td>o ");IntToStr(MaxAirSpeed, dyna) ;len += putString(dyna) ; len += putString("</td>");      //air speed

      len += putConstString("<td>");len += putString(Dlogo);len += putString("</td>");
                                        // humidity
      len += putConstString("<td>");//ByteToStr(integHum,txt);len += putString(txt);
             len += putString("----");//ByteToStr(decimHum,txt);len += putString(txt);
             len += putString(" %</td> </tr>");         // humidity
      len += putConstString("<tr bordercolor=\"#0000CC\" style=\"font: normal  bold 14pt Arial; text-align: center\"><td>  </td> <td> ");
             IntToStr(AverageAirSpeed, dyna) ;len += putString(dyna) ;len += putConstString(" Km/H</td><td>  </td><td>  </td></tr>");
      len += putConstString("<tr bordercolor=\"#0000CC\" style=\"font: normal  bold 14pt Arial; text-align: center\"><td>  </td> <td>o "); IntToStr(MinAirSpeed, dyna) ;len += putString(dyna) ;
             len += putConstString(" </td> <td>  </td> <td>  </td> </tr>");
      
      len += putConstString("</table>");
      len += putConstString("<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" height=\"30\" align=\"center\">");
      len += putConstString("<tr  bgcolor=\"#948ACC\" style=\"font: normal 12pt courier; text-align: center\">");
      for (i=0;i<15;i++){
         len += putConstString("<td>");  IntToStr(LastQuarterSpeed[i],dyna);  len += putString(dyna) ; len += putString("</td>");
         }
      len += putConstString("</tr><tr><td>@Kalarakis.gr</td></tr></table>");
      len += putConstString("</body></html>");
  }

  return(len) ;                               // return to the library with the number of bytes to transmit
}

/*
 * this function is called by the library
 * the user accesses to the UDP request by successive calls to Spi_Ethernet_getByte()
 * the user puts data in the transmit buffer by successive calls to Spi_Ethernet_putByte()
 * the function must return the length in bytes of the UDP reply, or 0 if nothing to transmit
 *
 * if you don't need to reply to UDP requests,
 * just define this function with a return(0) as single statement
 *
 */
unsigned int  SPI_Ethernet_UserUDP(unsigned char *remoteHost, unsigned int remotePort,
                                   unsigned int destPort, unsigned int reqLength, TEthPktFlags *flags) {
   return 0;              // back to the library with the length of the UDP reply
}






 void ReadValues(){
    //read values from port B  --> RB0,RB1,RB2,RB3 and put them to Aleds[4]
    ALeds[0]=ADC1_Read(0);
    ALeds[1]=ADC1_Read(1);
    ALeds[2]=ADC1_Read(2);
    ALeds[3]=ADC1_Read(3);
 
 }
 
void  NormalizeValues(){
     // values >2000=2000 >1000=1000 <299=200 300>=500
     int i;
     int Temp;
     for (i=0;i<5;i++){
        Temp=Aleds[i];

           if(Temp>2000) {
              ALeds[i]=2000;
              continue;
              }
           if (Temp>=1000 && Temp<=1990) {
               ALeds[i]=1000;
               continue;
               }
           if (Temp>=300 && Temp<=999) {
               ALeds[i]=500;
               continue;
               }
           if(Temp<=299) {
              ALeds[i]=200;
              continue;
              }
     }

}

void SetAirDirection(){
      if ((ALeds[0]==500) && (ALeds[1]==200) && (ALeds[2]) ==500 && (ALeds[3]==2000)){strcpy(Dlogo," B  ");}
      if ((ALeds[0]==500) && (ALeds[1]==200) && (ALeds[2]) ==1000 && (ALeds[3]==2000)){strcpy(Dlogo,"B-A");}
      if (ALeds[0]==200 && ALeds[1]==200 && ALeds[2] ==2000 && ALeds[3]==2000){strcpy(Dlogo," BA ");}
      if (ALeds[0]==200  && ALeds[1]==200 && ALeds[2] ==2000 && ALeds[3]==1000){strcpy(Dlogo,"BA-");}
      
      if (ALeds[0]==200 && ALeds[1]==500 && ALeds[2] ==2000 && ALeds[3]==500) {strcpy(Dlogo," A  ");}
      
      if (ALeds[0]==200  && ALeds[1]==1000 && ALeds[2] ==2000 && ALeds[3]==500){ strcpy(Dlogo,"A-NA");}

      if (ALeds[0]==200  && ALeds[1]==2000 && ALeds[2] ==2000 && ALeds[3]==200){strcpy(Dlogo," NA");}
      if (ALeds[0]==500  && ALeds[1]==2000 && ALeds[2] ==2000 && ALeds[3]==200){strcpy(Dlogo,"NA-N");}
      if (ALeds[0]==500  && ALeds[1]==2000 && ALeds[2] ==1000 && ALeds[3]==200){strcpy(Dlogo,"N-NA");}
      if (ALeds[0]==500  && ALeds[1]==2000 && ALeds[2] ==500 && ALeds[3]==200) {strcpy(Dlogo," N  ");}
      
      if (ALeds[0]==1000  && ALeds[1]==2000 && ALeds[2] ==500 && ALeds[3]==200){strcpy(Dlogo,"N-N");}
      if (ALeds[0]==2000  && ALeds[1]==2000 && ALeds[2] ==500 && ALeds[3]==200){strcpy(Dlogo,"  ");}
      if (ALeds[0]==2000  && ALeds[1]==2000 && ALeds[2] ==500 && ALeds[3]==500) {strcpy(Dlogo,"-");}
      
      if (ALeds[0]==2000  && ALeds[1]==500 && ALeds[2] ==500 && ALeds[3]==500)  {strcpy(Dlogo,"  ");}
      if (ALeds[0]==2000  && ALeds[1]==500 && ALeds[2] ==500 && ALeds[3]==1000) {strcpy(Dlogo,"-");}
      if (ALeds[0]==2000  && ALeds[1]==200 && ALeds[2] ==200 && ALeds[3]==2000){strcpy(Dlogo,"  ");}
      if (ALeds[0]==2000  && ALeds[1]==200 && ALeds[2] ==500 && ALeds[3]==2000){strcpy(Dlogo,"-");}
      if (ALeds[0]==1000  && ALeds[1]==200 && ALeds[2] ==500 && ALeds[3]==2000){strcpy(Dlogo,"-");}
}



void onTimer1Interupt() org 0x00001A  {
   int i;
   IFS0bits.T1IF = 0;                // clear interrupt flag
   PORTDbits.RD3 = ~PORTDbits.RD3 ;
   TickCounter++;
   TempTick++;
   
   asm CLRWDT;       // clear WDT Timer
   
   //every 4 tick read values and then a) normalize them , put apropriate values to arguments for the web page
   if (TickCounter>4){
      ReadValues();
      NormalizeValues();
      SetAirDirection();
      
      AirSpeed=AirSpeedTick * 10 /AirSpeedCoficient;    // 9 turns/sec = 10Km/H
      // alter max and min air speed acording prevousAirSpeed
      if (Airspeed >PreviusAirSpeed) {maxAirSpeed=(maxAirSpeed + Airspeed) /2;}
      if (AirSpeed < PreviusAirSpeed) {MinAirSpeed = (MinAirSpeed + AirSpeed) /2;}
      
        if (Airspeed >maxAirSpeed) {maxAirSpeed=Airspeed;}
        if (AirSpeed < MinAirSpeed) {MinAirSpeed = AirSpeed  ;}
        
      AverageAirSpeed=(PreviusAirSpeed + Airspeed)/2;
      if (AverageAirSpeed==0) {maxAirSpeed=maxAirspeed /2;}
      PreviusAirspeed=AirSpeed;

      integTemp=ADC1_Read(8) /59;
      if (TempTick>240){            // one minute passed
        // shift right 14 previous values
          for (i=0;i<14;i++){
             LastQuarterSpeed[i]=  LastQuarterSpeed[i+1];
          }
          LastQuarterSpeed[14]= AverageAirSpeed;
        TempTick=0;
        }
      AirSpeedTick=0;
      TickCounter=0;
      }
      
}

  void OnINT0_Interupt() org 0x000014 {
      IFS0bits.INT0IF =0;    //clear int0 flag
      AirSpeedTick++;    // count how many turns are done

 }
/**************************************************************************************************
* MAIN PROGRAM
**************************************************************************************************/
void main()
{   int i;
        ADPCFG |= 0xFF00;         // 0-8 AS ADC

        PORTB = 0;
        TRISB = 0xFFFF;           // set PORTB as input for buttons and adc

        PORTD = 0;
        TRISD = 0;                // set PORTD as output,

        ADC1_Init();              // Enable ADC module

        AirSpeedCoficient=EEPROM_Read(eeAddr);      // read from eeprom the stored Air speed coficient
        if (AirSpeedCoficient <= 0){AirSpeedCoficient=10;}   // if not yet set , give a default value
        for (i=0;i<14;i++){         // initialize
             LastQuarterSpeed[i]= 0;
          }
  /*
   * starts ENC28J60 with :
   * reset bit on RE1
   * CS bit on RE0
   * my MAC & IP address
   * full duplex
   */
  SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_5, _SPI_PRESCALE_PRI_16, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);
  SPI_Ethernet_Init(myMacAddr, myIpAddr, Spi_Ethernet_FULLDUPLEX) ;


   
          //*****************************************************************************
        /*The input clock (FOSC/4 or external clock) to the 16-bit
          Timer has a prescale option of 1:1, 1:8, 1:64 and 1:256,
          selected by control bits, TCKPS<1:0> (T1CON<5:4>).
          64000000/4  --> /256 ---> T=1/F sec
                      = 16000000 /256=   62500 --->    0.16us is the duration for 1 of PR1
                      250ms / 0.16us = 15625
                      500ms -----> 31250
        */
        TMR1 = 0;                                // clear timer 1
        PR1 =15625;                        // interrupt every 250ms
        IFS0bits.T1IF = 0;                // clr interrupt flag
        IEC0bits.T1IE = 1;                // set interrupt enable bit
        T1CONbits.TCKPS=3;                // prescaler 1:256
        T1CONbits.TON=1;                  //  start TMR1

 //*****************************************************************************
    //ENABLE INT1 connected to clock negative edge
         INTCON2bits.INT0EP=1; // enable int0
         IEC0bits.INT0IE =1;   // start int0

  while(1) {                            // do forever
    /*
     * if necessary, test the return value to get error code
     */
    SPI_Ethernet_doPacket() ;   // process incoming Ethernet packets

    /*
     * add your stuff here if needed
     * Spi_Ethernet_doPacket() must be called as often as possible
     * otherwise packets could be lost
     */
  }
}

/**************************************************************************************************
* End of File
**************************************************************************************************/